/*---[ druid.c ]------------------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The druid/wizard functions
 *--------------------------------------------------------------------*/

#include "scriptwriter.h"
#include <libgnomeui/gnome-window-icon.h>

GtkWidget *app;

void
quit_druid (void)
{
	gtk_widget_destroy (app);
	gtk_widget_destroyed (NULL, &app);
}

/*---[ box_sensitive ]------------------------------------------------
 * Flips the sensitivity of a given box, and all widgets within
 *--------------------------------------------------------------------*/
static void
box_sensitive (GtkWidget *box)
{
	if (GTK_WIDGET_IS_SENSITIVE (box))
		gtk_widget_set_sensitive (box, FALSE);
	else
		gtk_widget_set_sensitive (box, TRUE);
}

/*---[ device_exists ]------------------------------------------------
 * If *name is a valid network device, it is added to *devicelist
 *--------------------------------------------------------------------*/
static GList*
device_exists (gchar *name, GList *devicelist)
{
	FILE *f = fopen ("/proc/net/dev", "r");
	gchar buf[512];
	gint l = strlen (name);
	gchar *bp;

	while (fgets (buf, 512, f) != NULL) {
		bp = buf;
		while (*bp == ' ')
			bp++;
		if (strncmp (bp, name, l) == 0 && bp[l] == ':') {
			fclose (f);
			devicelist = g_list_append (devicelist, name);
			return (devicelist);
		}
	}
	fclose(f);
	return (devicelist);
}

/*---[ masq_next ]----------------------------------------------------
 * Called when the next button is pressed on the masq page
 *--------------------------------------------------------------------*/
static gint
masq_next (GnomeDruidPage *druidpage, gpointer druid, Druid *data)
{
	gchar *device = gtk_entry_get_text (GTK_ENTRY (data->device));
	gchar *intdevice = gtk_entry_get_text (GTK_ENTRY (data->intdevice));
	gboolean masq = GTK_WIDGET_IS_SENSITIVE (data->masq);

	if (masq && !g_strcasecmp (device, intdevice)) {
		GtkWidget *dialog=gnome_error_dialog _("The external and "
			"internal network interfaces\ncan't be the same. "
			"Please change one of them,\nor alternatively disable "
			"IP Masquerading.");
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		return TRUE;
	}
	return FALSE;
}

/*---[ druid_masq_page ]----------------------------------------------
 * Create the contents of the ipmasq setup page
 *--------------------------------------------------------------------*/
static GtkWidget*
druid_masq_page (Druid *data)
{
	GtkWidget *table;
	GtkWidget *table2;
	GtkWidget *label;
	GtkWidget *radio1;
	GtkWidget *radio2;
	GtkWidget *radio3;
	GtkWidget *entry;
	GtkWidget *combo;
	GList *devicelist = NULL;

	devicelist = device_exists ("ippp0", devicelist);
	devicelist = device_exists ("ippp1", devicelist);
	devicelist = device_exists ("ppp0", devicelist);
	devicelist = device_exists ("ppp1", devicelist);
	devicelist = device_exists ("eth0", devicelist);
	devicelist = device_exists ("eth1", devicelist);

	table = gtk_table_new (2, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	label = gtk_label_new _("Do you use IP Masquerading on your LAN, "
		"allowing all the machines on your network\nto communicate "
		"with the Internet via a single IP address?");

	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
	gtk_table_attach (GTK_TABLE (table), label, 0, 2, 0, 1,
		GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	radio1 = gtk_radio_button_new_with_label (NULL,
		_("No, I do not use IP Masquerading on my LAN"));
	gtk_table_attach (GTK_TABLE (table), radio1, 0, 2, 1, 2,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	radio2 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio1),
		_("Yes, this machine translates the IP addresses of my LAN "
		  "for the Internet"));
	gtk_table_attach (GTK_TABLE (table), radio2, 0, 2, 2, 3,
		GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	data->masq = gtk_hbox_new (FALSE, 10);
	gtk_widget_set_sensitive (data->masq, 0);
	gtk_table_attach (GTK_TABLE (table), data->masq, 0, 2, 3, 4,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	gtk_signal_connect_object (GTK_OBJECT (radio2), "toggled",
		GTK_SIGNAL_FUNC (box_sensitive), GTK_OBJECT (data->masq));

	table2 = gtk_table_new (2, 5, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE(table2), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE(table2), GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (data->masq), table2, FALSE, FALSE, 0);

 	label = gtk_label_new _("Select your internal network interface:");
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
	gtk_table_attach (GTK_TABLE (table2), label, 0, 1, 0, 1,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	combo = gtk_combo_new();
	gtk_combo_set_popdown_strings (GTK_COMBO (combo), devicelist);
	gtk_table_attach (GTK_TABLE (table2), combo, 1, 2, 0, 1,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	radio1 = gtk_radio_button_new_with_label (NULL,
		_("Autodetect internal network IP range"));
	gtk_table_attach (GTK_TABLE (table2), radio1, 0, 2, 1, 2,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	radio3 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio1),
		_("Specify internal network IP range manually"));
	gtk_table_attach (GTK_TABLE (table2), radio3, 0, 2, 2, 3,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	data->auto_int_ip = gtk_table_new (2, 2, FALSE);
	gtk_widget_set_sensitive (data->auto_int_ip, 0);
	gtk_table_attach (GTK_TABLE (table2), data->auto_int_ip, 0, 2, 3, 4,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	gtk_signal_connect_object (GTK_OBJECT (radio3), "toggled",
		GTK_SIGNAL_FUNC (box_sensitive), GTK_OBJECT (data->auto_int_ip));

 	label = gtk_label_new _("Enter the your internal network address range:");
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
	gtk_table_attach (GTK_TABLE (data->auto_int_ip), label, 0, 1, 0, 1,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	entry = gtk_entry_new ();
	gtk_entry_set_text (GTK_ENTRY (entry), "192.168.1.0/24");
	gtk_table_attach (GTK_TABLE (data->auto_int_ip), entry, 1, 2, 0, 1,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

 	label = gtk_label_new _("The network range is given in the form of "
				"addess/mask. 192.168.1.0/24 is the common\n"
				"IP range reserved for internal networks and "
				"should work for most people.");
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
	gtk_table_attach (GTK_TABLE (data->auto_int_ip), label, 0, 2, 1, 2,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	data->intdevice = (GTK_COMBO (combo)->entry);
	data->intrange = entry;

	gtk_widget_show_all (table);
	return table;
}

/*---[ druid_device_page ]--------------------------------------------
 * Create the contents of the device selection page
 *--------------------------------------------------------------------*/
static GtkWidget*
druid_device_page (Druid *data)
{
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *hbox;
	GtkWidget *combo;
	GList *devicelist = NULL;

	devicelist = device_exists ("ippp0", devicelist);
	devicelist = device_exists ("ippp1", devicelist);
	devicelist = device_exists ("ppp0", devicelist);
	devicelist = device_exists ("ppp1", devicelist);
	devicelist = device_exists ("eth0", devicelist);
	devicelist = device_exists ("eth1", devicelist);

	table = gtk_table_new (1, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);

	label = gtk_label_new _("Choose the device you connect to the "
		"Internet with.\n\nIf you connect to the net by modem, "
		"your device name is likely ppp0. If you have an\nISDN "
		"line or an ethernet connection, choose ippp0 or eth0 "
		"respectively.");
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	hbox = gtk_hbox_new (FALSE, 1);
	gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 1, 2,
		GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

 	label = gtk_label_new _("Detected device(s):");
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);

	combo = gtk_combo_new();
	gtk_combo_set_popdown_strings (GTK_COMBO (combo), devicelist);
	gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 5);

	data->device = (GTK_COMBO (combo)->entry);

	gtk_widget_show_all (table);
	return table;
}

/*---[ druid_services_page ]------------------------------------------
 * Create the contents of the service selection page
 *--------------------------------------------------------------------*/
static GtkWidget*
druid_services_page (Druid *data)
{
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *vbox;
	GtkWidget *radio1, *radio2;
		
	table = gtk_table_new (4, 1, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);

	label = gtk_label_new (_("Do you run server software on your machine "
			       "and want other people to have access\nto the "
			       "network services you provide?"));
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	radio1 = gtk_radio_button_new_with_label (NULL,
		_("No, I do not run any public network services on my machine"));
	gtk_table_attach (GTK_TABLE (table), radio1, 0, 1, 1, 2,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	radio2 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (radio1),
		_("Yes, I run the following services on my machine:"));
	gtk_table_attach (GTK_TABLE (table), radio2, 0, 1, 2, 3,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	data->services = gtk_hbox_new (FALSE, 10);
	gtk_widget_set_sensitive (data->services, 0);
	gtk_table_attach (GTK_TABLE (table), data->services, 0, 1, 3, 4,
		GTK_FILL, GTK_FILL, GNOME_PAD, 0);

	gtk_signal_connect_object (GTK_OBJECT (radio2), "toggled",
		GTK_SIGNAL_FUNC (box_sensitive), GTK_OBJECT (data->services));

	/* Column 1 */
	vbox = gtk_vbox_new (FALSE, 1);

	data->ftp = gtk_check_button_new_with_label ("FTP");
	gtk_box_pack_start (GTK_BOX (vbox), data->ftp, FALSE, FALSE, 0);

	data->ssh = gtk_check_button_new_with_label ("SSH");
	gtk_box_pack_start (GTK_BOX (vbox), data->ssh, FALSE, FALSE, 0);

	data->telnet = gtk_check_button_new_with_label ("Telnet");
	gtk_box_pack_start (GTK_BOX (vbox), data->telnet, FALSE, FALSE, 0);

	data->smtp = gtk_check_button_new_with_label ("SMTP");
	gtk_box_pack_start (GTK_BOX (vbox), data->smtp, FALSE, FALSE, 0);

	data->dns = gtk_check_button_new_with_label ("DNS");
	gtk_box_pack_start (GTK_BOX (vbox), data->dns, FALSE, FALSE, 0);

	data->finger = gtk_check_button_new_with_label ("Finger");
	gtk_box_pack_start (GTK_BOX (vbox), data->finger, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (data->services), vbox, FALSE, FALSE, 0);

	/* Column 2 */
	vbox = gtk_vbox_new (FALSE, 1);

	data->www = gtk_check_button_new_with_label ("WWW");
	gtk_box_pack_start (GTK_BOX (vbox), data->www, FALSE, FALSE, 0);

	data->sslweb = gtk_check_button_new_with_label ("SSL web");
	gtk_box_pack_start (GTK_BOX (vbox), data->sslweb, FALSE, FALSE, 0);

	data->pop = gtk_check_button_new_with_label ("POP");
	gtk_box_pack_start (GTK_BOX (vbox), data->pop, FALSE, FALSE, 0);

	data->imap = gtk_check_button_new_with_label ("IMAP");
	gtk_box_pack_start (GTK_BOX (vbox), data->imap, FALSE, FALSE, 0);

	data->ident = gtk_check_button_new_with_label ("IDENT");
	gtk_box_pack_start (GTK_BOX (vbox), data->ident, FALSE, FALSE, 0);

	data->nntp = gtk_check_button_new_with_label ("NNTP");
	gtk_box_pack_start (GTK_BOX (vbox), data->nntp, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (data->services), vbox, FALSE, FALSE, 0);

	/* Column 3 */
	vbox = gtk_vbox_new (FALSE, 1);

	data->ntp = gtk_check_button_new_with_label ("NTP");
	gtk_box_pack_start (GTK_BOX (vbox), data->ntp, FALSE, FALSE, 0);

	data->samba = gtk_check_button_new_with_label ("SAMBA/NetBios");
	gtk_box_pack_start (GTK_BOX (vbox), data->samba, FALSE, FALSE, 0);

	data->routed = gtk_check_button_new_with_label ("Routed");
	gtk_box_pack_start (GTK_BOX (vbox), data->routed, FALSE, FALSE, 0);

	data->nfs = gtk_check_button_new_with_label ("NFS");
	gtk_box_pack_start (GTK_BOX (vbox), data->nfs, FALSE, FALSE, 0);

	data->x = gtk_check_button_new_with_label ("Xwindows");
	gtk_box_pack_start (GTK_BOX (vbox), data->x, FALSE, FALSE, 0);

	data->dhcp = gtk_check_button_new_with_label ("DHCP");
	gtk_box_pack_start (GTK_BOX (vbox), data->dhcp, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (data->services), vbox, FALSE, FALSE, 0);

	gtk_widget_show_all (table);

	return table;
}

/*---[ rundruid ]-----------------------------------------------------
 * The main druid function
 *--------------------------------------------------------------------*/
void
rundruid (void)
{
	GtkWidget *druid;
	GtkWidget *startpage;
	GtkWidget *devicepage;
	GtkWidget *masqpage;
	GtkWidget *servicespage;	
	GtkWidget *finishpage;
	Druid *data;
	GdkColor color;

	/* Check that the window is not already created */
	if (app != NULL) {
		g_assert (GTK_WIDGET_REALIZED (app));
		gdk_window_show (app->window);
		gdk_window_raise (app->window);
		return;
	}

	/* Check that the user is a superuser */
	if (geteuid ()) {
		GtkWidget *dialog=gnome_error_dialog _(
			"You must be root to use the Firestarter wizard.");
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		return;
	}

	app = gnome_app_new ("firestarter", _("Firestarter Firewall Wizard"));
	gtk_window_set_position (GTK_WINDOW (app), GTK_WIN_POS_CENTER_ALWAYS);
	gtk_signal_connect (GTK_OBJECT (app), "destroy",
			    GTK_SIGNAL_FUNC (quit_druid), NULL);

	gnome_window_icon_set_from_file (GTK_WINDOW (app),
		"/usr/share/pixmaps/firestarter_druid.xpm");

	data = g_new(Druid, 1);

	druid = gnome_druid_new ();
	gtk_signal_connect (GTK_OBJECT (druid), "cancel",
			    GTK_SIGNAL_FUNC (quit_druid), NULL);
	gnome_app_set_contents (GNOME_APP (app), druid);

	/* Create the basic druid pages */	
	startpage = gnome_druid_page_start_new_with_vals (_("Firestarter Wizard"),
		_("Welcome to the Firestarter Firewall Wizard\n\n"
		"This wizard will help you set up a customized packet "
		"filtering firewall\nfor your Linux machine in just a "
		"few easy steps.\n\n\n"),
		  NULL, NULL);

	devicepage = gnome_druid_page_standard_new_with_vals (_("Choose your external interface"), NULL);
	masqpage = gnome_druid_page_standard_new_with_vals (_("IP Masquerade setup"), NULL);
	servicespage = gnome_druid_page_standard_new_with_vals (_("Choose allowed services"), NULL);

	finishpage = gnome_druid_page_finish_new_with_vals (_("Wizard done"),
		_("The Firestarter wizard is done gathering the necessary "
		"data.\nClick the finish button to start the firewall.\n\n"),
		NULL, NULL);
	
	gtk_signal_connect (GTK_OBJECT (finishpage), "finish",
			    GTK_SIGNAL_FUNC (write_script), data);

	gtk_signal_connect (GTK_OBJECT (masqpage), "next",
			    GTK_SIGNAL_FUNC (masq_next), data);
						
	/* Set the color of the various druid page elements */
	color.red = 0*255;
	color.green = 0*255;
	color.blue = 0*255;	
	
	gnome_druid_page_start_set_bg_color (GNOME_DRUID_PAGE_START (startpage), &color);	
	gnome_druid_page_finish_set_bg_color (GNOME_DRUID_PAGE_FINISH (finishpage), &color);	
	
	gnome_druid_page_standard_set_bg_color (GNOME_DRUID_PAGE_STANDARD (devicepage), &color);
	gnome_druid_page_standard_set_bg_color (GNOME_DRUID_PAGE_STANDARD (masqpage), &color);
	gnome_druid_page_standard_set_bg_color (GNOME_DRUID_PAGE_STANDARD (servicespage), &color);
	
	gnome_druid_page_standard_set_logo_bg_color (GNOME_DRUID_PAGE_STANDARD (devicepage), &color);
	gnome_druid_page_standard_set_logo_bg_color (GNOME_DRUID_PAGE_STANDARD (masqpage), &color);
	gnome_druid_page_standard_set_logo_bg_color (GNOME_DRUID_PAGE_STANDARD (servicespage), &color);
		
	gnome_druid_page_start_set_logo_bg_color (GNOME_DRUID_PAGE_START (startpage), &color);
	gnome_druid_page_finish_set_logo_bg_color (GNOME_DRUID_PAGE_FINISH (finishpage), &color);
			
	gnome_druid_append_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (startpage));
	gnome_druid_append_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (devicepage));
	gnome_druid_append_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (masqpage));
	gnome_druid_append_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (servicespage));
	gnome_druid_append_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (finishpage));

	/* Pack the tables returned from the content creation fuctions into the emty druid pages */
	gtk_box_pack_start (GTK_BOX (GNOME_DRUID_PAGE_STANDARD (devicepage)->vbox),
			    		druid_device_page (data), TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (GNOME_DRUID_PAGE_STANDARD (masqpage)->vbox),
			    		druid_masq_page (data), TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (GNOME_DRUID_PAGE_STANDARD (servicespage)->vbox),
			    		druid_services_page (data), TRUE, TRUE, 0);
	/* Page to start from */	
	gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (startpage));

	gtk_widget_show_all (app);
	
	gnome_config_set_bool ("/firestarter/Druid/firsttime", FALSE);
	gnome_config_sync ();
}